# 测试python访问数据库
import pymssql

# 取得连接时会稍微等待一小段时间
from time import sleep

# 输出结果时用到，将检索的结果保存成csv
import pandas as pd

import sys

# 取得sql文件名称
sql_file = "query" + sys.argv[2] + ".sql"
user_output_txt_file = "user_output" + sys.argv[2] + ".txt"
user_output_csv_file = "user_output" + sys.argv[2] + ".csv"
# 验证类型：本地--local_test，线上--ci_test
test_type = sys.argv[1]

# 可能连接数据库时需要消耗一点时间，这里用循环的方式
def get_conn():
  # 循环3次，连接数据库
  for i in range(0, 2):
   try:
       # 调用远程driver，获取成功即返回
       if i==0:
         # 等待5秒后重新获取
         sleep(5)
       elif i == 1:
         # 等待10秒后重新获取
         sleep(10)
       else:
         # 等待15秒后重新获取
         sleep(15)
       # 根据平台不同采用不同的连接方式
       if sys.platform == 'linux':
        # linux操作系统，用户名密码登录
        conn = pymssql.connect(host='localhost', user='SA', password='<MyStrong@Passw0rd>')   
       else:
        # windows操作系统，连接端口号
        conn = pymssql.connect(host='localhost', server='localhost\SQLEXPRESS', port='55895', database='master')
       return conn
   except Exception:
     print ("Waiting,connect database...")
  # 没有启动远程driver，程序终止
  print ("connect database failed！")
  exit(1)

# 将sql文件的内容转换成字符串
def parse_sql(filename):
    # 读取 sql 文件文本内容，sqltxt 为 list 类型
    sql = open(filename, 'r', encoding = 'utf-8')
    sqltxt = sql.readlines()
 
    # 读取之后关闭文件
    sql.close()

    # list 转 str
    sql = "".join(sqltxt)

    return sql

# 主函数
if __name__ == '__main__':

    # 调用get_conn来取得数据库连接并获取cursor
    conn = get_conn()
    cursor = conn.cursor()

    # 只有本地验证第一次运行case或者线上验证时才建库、建表
    if (int(sys.argv[2]) == 1 and test_type == "local_test") or test_type == "ci_test":
        # 创建测试数据库
        createdb_sql = parse_sql('createdb.sql')
        conn.autocommit(True)
        cursor.execute(createdb_sql)
        conn.autocommit(False)

        # 创建数据库表
        createtable_sql = parse_sql('createtable.sql')
        cursor.execute(createtable_sql)
        conn.commit()

    # 运行学生代码
    query_sql = parse_sql(sql_file)
    df = pd.read_sql(query_sql, conn)
    # 保存成csv格式，带表头，带每行前的序号，注意编码方式
    # csv方便用户查看
    df.to_csv(user_output_csv_file, index=True, header=True, encoding='gb18030')
    # txt用于和答案模板比较
    df.to_csv(user_output_txt_file, index=False, header=True, encoding='UTF-8')

    # 连接用完后记得关闭以释放资源
    conn.close()
